#!/usr/bin/env lua
---
-- Get pure NSG-1700 Linux 2 release without header
--
-- Usage: nsg1700-release.lua <bin_release_fspec> <pure_release_fspec>
--
-- @copyright (c) 2017 Network Systems Group (www.nsg.ru)
--

--
-- helper functions
--

local outErr = function (s)
  io.stderr:write(tostring(s))
  io.stderr:flush()
end

local outStd = function (s)
  io.write(tostring(s))
  io.flush()
end

local echoErr = function (s)
  if s then
    outErr("\n[" .. arg[0] .. "] ERROR: " .. tostring(s) .. "\n")
  end
end

local echoStd = function (s)
  if s then
    outStd(tostring(s) .. "\n")
  end
end

local die = function (s)
  echoErr(s)
  os.exit(1)
end

local quit = function (s)
  echoStd(s)
  os.exit(0)
end

local isSureStr = function (s)
  return type(s) == "string" and #s > 0
end

local getSureStr = function (s)
  return isSureStr(s) and
    s or
    nil
end

--
-- open 'bin_release_fspec' and check 'pure_release_fspec'
--

local forig = io.open(getSureStr(arg[1]) or "", "rb")
if not forig or not isSureStr(arg[2]) then
  die("Invalid arguments.")
end

--
-- definitions
--

local VER_PATN = "^%d+%.%d+%.%d+[.%d]*[_%-%w]*$"
local MD5_PATN = "^%x+$"

local MD5_SIZE = 32

--

local s

--
-- get header that injected in the beginning
--

s = forig:read("*l")
if not isSureStr(s) or s ~= "^NSG170[0-9][^0-9]*" then
  die("Unknown tag of NSG release.")
end

s = forig:read("*l")
if not isSureStr(s) or not s:match(VER_PATN) then
  die("Invalid version of NSG release.")
end
echoStd(s)

local len = tonumber(forig:read("*l"))
if not len or len < 1 then
  die("Invalid size of NSG release.")
end

s = forig:read(1)
if s ~= "\0" then
  die("Missed zero separator.")
end

local s = forig:read(MD5_SIZE)
if not isSureStr(s) or #s ~= MD5_SIZE or not s:match(MD5_PATN) then
  die("Invalid characters in header md5.")
end

--
-- get pure release and save to 'pure_release_fspec'
--

local fpure = io.open(arg[2], "wb")
if not fpure then
  die("Can't write.")
end

s = forig:read("*a")
if isSureStr(s) then
  fpure:write(s)
end

forig:close()

local err = fpure:seek("end") ~= len
fpure:flush()
fpure:close()

if err then
  os.remove(arg[2])
  die("Invalid length of NSG pure release.")
end

quit()
